@@ -7,19 +7,22 @@ from astropy.coordinates import Angle, SkyCoord
 from astropy.io import fits
 from astropy.nddata import Cutout2D
 from astropy.wcs import WCS
-from astropy.wcs.utils import proj_plane_pixel_scales
+from astropy.wcs.utils import (
+    celestial_frame_to_wcs,
+    proj_plane_pixel_scales,
+    wcs_to_celestial_frame,
+)
 from regions import SkyRegion
 from .geom import (
     Geom,
     MapCoord,
-    axes_pix_to_coord,
     find_and_read_bands,
     get_shape,
     make_axes,
     pix_tuple_to_idx,
     skycoord_to_lonlat,
 )
-from .utils import INVALID_INDEX
+from .utils import INVALID_INDEX, slice_to_str, str_to_slice
 
 __all__ = ["WcsGeom"]
 
@@ -78,87 +81,6 @@ def get_resampled_wcs(wcs, factor, downsampled):
     return wcs
 
 
-# TODO: remove this function, move code to the one caller below
-def _make_image_header(
-    nxpix=100,
-    nypix=100,
-    binsz=0.1,
-    xref=0,
-    yref=0,
-    proj="CAR",
-    coordsys="GAL",
-    xrefpix=None,
-    yrefpix=None,
-):
-    """Generate a FITS header from scratch.
-
-    Uses the same parameter names as the Fermi tool gtbin.
-
-    If no reference pixel position is given it is assumed ot be
-    at the center of the image.
-
-    Parameters
-    ----------
-    nxpix : int, optional
-        Number of pixels in x axis. Default is 100.
-    nypix : int, optional
-        Number of pixels in y axis. Default is 100.
-    binsz : float, optional
-        Bin size for x and y axes in units of degrees. Default is 0.1.
-    xref : float, optional
-        Coordinate system value at reference pixel for x axis. Default is 0.
-    yref : float, optional
-        Coordinate system value at reference pixel for y axis. Default is 0.
-    proj : string, optional
-        Projection type. Default is 'CAR' (cartesian).
-    coordsys : {'CEL', 'GAL'}, optional
-        Coordinate system. Default is 'GAL' (Galactic).
-    xrefpix : float, optional
-        Coordinate system reference pixel for x axis. Default is None.
-    yrefpix: float, optional
-        Coordinate system reference pixel for y axis. Default is None.
-
-    Returns
-    -------
-    header : `~astropy.io.fits.Header`
-        Header
-    """
-    nxpix = int(nxpix)
-    nypix = int(nypix)
-    if not xrefpix:
-        xrefpix = (nxpix + 1) / 2.0
-    if not yrefpix:
-        yrefpix = (nypix + 1) / 2.0
-
-    if coordsys == "CEL":
-        ctype1, ctype2 = "RA---", "DEC--"
-    elif coordsys == "GAL":
-        ctype1, ctype2 = "GLON-", "GLAT-"
-    else:
-        raise ValueError(f"Unsupported coordsys: {coordsys!r}")
-
-    pars = {
-        "NAXIS": 2,
-        "NAXIS1": nxpix,
-        "NAXIS2": nypix,
-        "CTYPE1": ctype1 + proj,
-        "CRVAL1": xref,
-        "CRPIX1": xrefpix,
-        "CUNIT1": "deg",
-        "CDELT1": -binsz,
-        "CTYPE2": ctype2 + proj,
-        "CRVAL2": yref,
-        "CRPIX2": yrefpix,
-        "CUNIT2": "deg",
-        "CDELT2": binsz,
-    }
-
-    header = fits.Header()
-    header.update(pars)
-
-    return header
-
-
 class WcsGeom(Geom):
     """Geometry class for WCS maps.
 
@@ -179,16 +101,18 @@ class WcsGeom(Geom):
         Reference pixel coordinate in each image plane.
     axes : list
         Axes for non-spatial dimensions
+    cutout_info : dict
+        Dict with cutout info, if the `WcsGeom` was created by `WcsGeom.cutout()`
     """
 
     _slice_spatial_axes = slice(0, 2)
     _slice_non_spatial_axes = slice(2, None)
     is_hpx = False
 
-    def __init__(self, wcs, npix, cdelt=None, crpix=None, axes=None):
+    def __init__(self, wcs, npix, cdelt=None, crpix=None, axes=None, cutout_info=None):
         self._wcs = wcs
-        self._coordsys = get_coordys(wcs)
-        self._projection = get_projection(wcs)
+        self._frame = wcs_to_celestial_frame(wcs).name
+        self._projection = wcs.wcs.ctype[0][5:]
         self._axes = make_axes(axes)
 
         if cdelt is None:
@@ -204,6 +128,13 @@ class WcsGeom(Geom):
             crpix = tuple(1.0 + (np.array(self._npix) - 1.0) / 2.0)
 
         self._crpix = crpix
+        self._cutout_info = cutout_info
+
+        # define cached methods
+        self.get_coord = lru_cache()(self.get_coord)
+        self.solid_angle = lru_cache()(self.solid_angle)
+        self.bin_volume = lru_cache()(self.bin_volume)
+        self.to_image = lru_cache()(self.to_image)
 
     @property
     def data_shape(self):
@@ -233,12 +164,17 @@ class WcsGeom(Geom):
         return self._wcs
 
     @property
-    def coordsys(self):
+    def frame(self):
         """Coordinate system of the projection.
 
-        Galactic ('GAL') or Equatorial ('CEL').
+        Galactic ("galactic") or Equatorial ("icrs").
         """
-        return self._coordsys
+        return self._frame
+
+    @property
+    def cutout_info(self):
+        """Cutout info dict."""
+        return self._cutout_info
 
     @property
     def projection(self):
@@ -342,7 +278,7 @@ class WcsGeom(Geom):
         npix=None,
         binsz=0.5,
         proj="CAR",
-        coordsys="CEL",
+        frame="icrs",
         refpix=None,
         axes=None,
         skydir=None,
@@ -379,8 +315,8 @@ class WcsGeom(Geom):
             Sky position of map center.  Can be either a SkyCoord
             object or a tuple of longitude and latitude in deg in the
             coordinate system of the map.
-        coordsys : {'CEL', 'GAL'}, optional
-            Coordinate system, either Galactic ('GAL') or Equatorial ('CEL').
+        frame : {"icrs", "galactic"}, optional
+            Coordinate system, either Galactic ("galactic") or Equatorial ("icrs").
         axes : list
             List of non-spatial axes.
         proj : string, optional
@@ -405,11 +341,12 @@ class WcsGeom(Geom):
         >>> geom = WcsGeom.create(npix=([100,200],[100,200]), binsz=0.1, axes=[axis])
         """
         if skydir is None:
-            xref, yref = (0.0, 0.0)
+            crval = (0.0, 0.0)
         elif isinstance(skydir, tuple):
-            xref, yref = skydir
+            crval = skydir
         elif isinstance(skydir, SkyCoord):
-            xref, yref, frame = skycoord_to_lonlat(skydir, coordsys=coordsys)
+            xref, yref, frame = skycoord_to_lonlat(skydir, frame=frame)
+            crval = (xref, yref)
         else:
             raise ValueError(f"Invalid type for skydir: {type(skydir)!r}")
 
@@ -433,20 +370,21 @@ class WcsGeom(Geom):
             npix = cast_to_shape(npix, shape, int)
 
         if refpix is None:
-            refpix = (None, None)
+            nxpix = int(npix[0].flat[0])
+            nypix = int(npix[1].flat[0])
+            refpix = ((nxpix + 1) / 2.0, (nypix + 1) / 2.0)
 
-        header = _make_image_header(
-            nxpix=npix[0].flat[0],
-            nypix=npix[1].flat[0],
-            binsz=binsz[0].flat[0],
-            xref=float(xref),
-            yref=float(yref),
-            proj=proj,
-            coordsys=coordsys,
-            xrefpix=refpix[0],
-            yrefpix=refpix[1],
-        )
-        wcs = WCS(header)
+        # get frame class
+        frame = SkyCoord(np.nan, np.nan, frame=frame, unit="deg").frame
+        wcs = celestial_frame_to_wcs(frame, projection=proj)
+        wcs.wcs.crpix = refpix
+        wcs.wcs.crval = crval
+
+        cdelt = binsz[0].flat[0]
+        wcs.wcs.cdelt = (-cdelt, cdelt)
+
+        wcs.array_shape = npix[0].flat[0], npix[1].flat[0]
+        wcs.wcs.datfix()
         return cls(wcs, npix, cdelt=binsz, axes=axes)
 
     @classmethod
@@ -465,10 +403,7 @@ class WcsGeom(Geom):
         wcs : `~WcsGeom`
             WCS geometry object.
         """
-        wcs = WCS(header)
-        naxis = wcs.naxis
-        for i in range(naxis - 2):
-            wcs = wcs.dropaxis(2)
+        wcs = WCS(header, naxis=(1, 2))
 
         axes = find_and_read_bands(hdu_bands)
         shape = tuple([ax.nbin for ax in axes])
@@ -486,7 +421,20 @@ class WcsGeom(Geom):
             npix = (header["NAXIS1"], header["NAXIS2"])
             cdelt = None
 
-        return cls(wcs, npix, cdelt=cdelt, axes=axes)
+        if "PSLICE1" in header:
+            cutout_info = {}
+            cutout_info["parent-slices"] = (
+                str_to_slice(header["PSLICE2"]),
+                str_to_slice(header["PSLICE1"]),
+            )
+            cutout_info["cutout-slices"] = (
+                str_to_slice(header["CSLICE2"]),
+                str_to_slice(header["CSLICE1"]),
+            )
+        else:
+            cutout_info = None
+
+        return cls(wcs, npix, cdelt=cdelt, axes=axes, cutout_info=cutout_info)
 
     def _make_bands_cols(self, hdu=None, conv=None):
 
@@ -529,6 +477,16 @@ class WcsGeom(Geom):
         for ax in self.axes:
             shape += f",{ax.nbin}"
         header["WCSSHAPE"] = f"({shape})"
+
+        if self.cutout_info is not None:
+            slices_parent = self.cutout_info["parent-slices"]
+            slices_cutout = self.cutout_info["cutout-slices"]
+
+            header["PSLICE1"] = (slice_to_str(slices_parent[1]), "Parent slice")
+            header["PSLICE2"] = (slice_to_str(slices_parent[0]), "Parent slice")
+            header["CSLICE1"] = (slice_to_str(slices_cutout[1]), "Cutout slice")
+            header["CSLICE2"] = (slice_to_str(slices_cutout[0]), "Cutout slice")
+
         return header
 
     def get_image_shape(self, idx):
@@ -538,7 +496,6 @@ class WcsGeom(Geom):
         else:
             return int(self.npix[0][idx]), int(self.npix[1][idx])
 
-    @lru_cache()
     def get_idx(self, idx=None, flat=False):
         pix = self.get_pix(idx=idx, mode="center")
         if flat:
@@ -565,7 +522,6 @@ class WcsGeom(Geom):
         pix = np.meshgrid(*pix[::-1], indexing="ij")[::-1]
         return pix
 
-    @lru_cache()
     def get_pix(self, idx=None, mode="center"):
         """Get map pix coordinates from the geometry.
 
@@ -586,8 +542,7 @@ class WcsGeom(Geom):
             _[~m] = INVALID_INDEX.float
         return pix
 
-    @lru_cache()
-    def get_coord(self, idx=None, flat=False, mode="center", coordsys=None):
+    def get_coord(self, idx=None, flat=False, mode="center", frame=None):
         """Get map coordinates from the geometry.
 
         Parameters
@@ -610,13 +565,13 @@ class WcsGeom(Geom):
         axes_names = ["lon", "lat"] + [ax.name for ax in self.axes]
         cdict = dict(zip(axes_names, coords))
 
-        if coordsys is None:
-            coordsys = self.coordsys
+        if frame is None:
+            frame = self.frame
 
-        return MapCoord.create(cdict, coordsys=self.coordsys).to_coordsys(coordsys)
+        return MapCoord.create(cdict, frame=self.frame).to_frame(frame)
 
     def coord_to_pix(self, coords):
-        coords = MapCoord.create(coords, coordsys=self.coordsys)
+        coords = MapCoord.create(coords, frame=self.frame)
 
         if coords.size == 0:
             return tuple([np.array([]) for i in range(coords.ndim)])
@@ -657,7 +612,9 @@ class WcsGeom(Geom):
             u.Quantity(coords[1], unit="deg", copy=False),
         ]
 
-        coords += axes_pix_to_coord(self.axes, pix[self._slice_non_spatial_axes])
+        for ax, t in zip(self.axes, pix[self._slice_non_spatial_axes]):
+            coords += [ax.pix_to_coord(t)]
+
         return tuple(coords)
 
     def pix_to_idx(self, pix, clip=False):
@@ -691,17 +648,86 @@ class WcsGeom(Geom):
         idx = self.coord_to_idx(coords)
         return np.all(np.stack([t != INVALID_INDEX.int for t in idx]), axis=0)
 
-    @lru_cache()
     def to_image(self):
         npix = (np.max(self._npix[0]), np.max(self._npix[1]))
         cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
-        return self.__class__(self._wcs, npix, cdelt=cdelt)
+        return self.__class__(
+            self._wcs, npix, cdelt=cdelt, cutout_info=self.cutout_info
+        )
 
     def to_cube(self, axes):
         npix = (np.max(self._npix[0]), np.max(self._npix[1]))
         cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
         axes = copy.deepcopy(self.axes) + axes
-        return self.__class__(self._wcs.deepcopy(), npix, cdelt=cdelt, axes=axes)
+        return self.__class__(
+            self._wcs.deepcopy(),
+            npix,
+            cdelt=cdelt,
+            axes=axes,
+            cutout_info=self.cutout_info,
+        )
+
+    def squash(self, axis):
+        """Squash geom axis.
+
+        Parameters
+        ----------
+        axis : str
+            Axis to squash.
+
+        Returns
+        -------
+        geom : `Geom`
+            Geom with squashed axis.
+        """
+        _ = self.get_axis_by_name(axis)
+        npix = (np.max(self._npix[0]), np.max(self._npix[1]))
+        cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
+
+        axes = []
+        for ax in copy.deepcopy(self.axes):
+            if ax.name == axis:
+                ax = ax.squash()
+            axes.append(ax)
+
+        return self.__class__(
+            self._wcs.deepcopy(),
+            npix,
+            cdelt=cdelt,
+            axes=axes,
+            cutout_info=self.cutout_info,
+        )
+
+    def drop(self, axis):
+        """Drop an axis from the geom.
+
+        Parameters
+        ----------
+        axis : str
+            Name of the axis to remove.
+
+        Returns
+            -------
+        geom : `Geom`
+            New geom with the axis removed.
+        """
+        _ = self.get_axis_by_name(axis)
+        npix = (np.max(self._npix[0]), np.max(self._npix[1]))
+        cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
+
+        axes = []
+        for ax in copy.deepcopy(self.axes):
+            if ax.name == axis:
+                continue
+            axes.append(ax)
+
+        return self.__class__(
+            self._wcs.deepcopy(),
+            npix,
+            cdelt=cdelt,
+            axes=axes,
+            cutout_info=self.cutout_info,
+        )
 
     def pad(self, pad_width):
         if np.isscalar(pad_width):
@@ -763,7 +789,7 @@ class WcsGeom(Geom):
             return self._init_copy(axes=axes)
 
     def to_binsz(self, binsz):
-        """Change pixel size of the geometry
+        """Change pixel size of the geometry.
 
         Parameters
         ----------
@@ -775,16 +801,15 @@ class WcsGeom(Geom):
         geom : `WcsGeom`
             Geometry with new pixel size.
         """
-        kwargs = {}
-        kwargs["skydir"] = self.center_skydir
-        kwargs["binsz"] = binsz
-        kwargs["width"] = self.width
-        kwargs["proj"] = self.projection
-        kwargs["coordsys"] = self.coordsys
-        kwargs["axes"] = copy.deepcopy(self.axes)
-        return self.create(**kwargs)
+        return self.create(
+            skydir=self.center_skydir,
+            binsz=binsz,
+            width=self.width,
+            proj=self.projection,
+            frame=self.frame,
+            axes=copy.deepcopy(self.axes),
+        )
 
-    @lru_cache()
     def solid_angle(self):
         """Solid angle array (`~astropy.units.Quantity` in ``sr``).
 
@@ -822,7 +847,6 @@ class WcsGeom(Geom):
 
         return u.Quantity(area_low_right + area_up_left, "sr", copy=False)
 
-    @lru_cache()
     def bin_volume(self):
         """Bin volume (`~astropy.units.Quantity`)"""
         bin_volume = self.to_image().solid_angle()
@@ -880,7 +904,14 @@ class WcsGeom(Geom):
             mode=mode,
         )
 
-        return self._init_copy(wcs=c2d.wcs, npix=c2d.shape[::-1])
+        cutout_info = {
+            "parent-slices": c2d.slices_original,
+            "cutout-slices": c2d.slices_cutout,
+        }
+
+        return self._init_copy(
+            wcs=c2d.wcs, npix=c2d.shape[::-1], cutout_info=cutout_info
+        )
 
     def region_mask(self, regions, inside=True):
         """Create a mask from a given list of regions
@@ -955,12 +986,34 @@ class WcsGeom(Geom):
             f"\taxes       : {axes}\n"
             f"\tshape      : {self.data_shape[::-1]}\n"
             f"\tndim       : {self.ndim}\n"
-            f"\tcoordsys   : {self.coordsys}\n"
+            f"\tframe   : {self.frame}\n"
             f"\tprojection : {self.projection}\n"
             f"\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n"
             f"\twidth      : {self.width[0][0]:.1f} x {self.width[1][0]:.1f}\n"
         )
 
+    def is_aligned(self, other, tolerance=1e-6):
+        """Check if WCS and extra axes are aligned.
+
+        Parameters
+        ----------
+        other : `WcsGeom`
+            Other geom.
+        tolerance : float
+            Tolerance for the comparison.
+
+        Returns
+        -------
+        aligned : bool
+            Whether geometries are aligned
+        """
+        for axis, otheraxis in zip(self.axes, other.axes):
+            if axis != otheraxis:
+                return False
+
+        # check WCS consistency with a priori tolerance of 1e-6
+        return self.wcs.wcs.compare(other.wcs.wcs, cmp=2, tolerance=tolerance)
+
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
@@ -974,7 +1027,9 @@ class WcsGeom(Geom):
                 return False
 
         # check WCS consistency with a priori tolerance of 1e-6
-        return self.wcs.wcs.compare(other.wcs.wcs, tolerance=1e-6)
+        # cmp=1 parameter ensures no comparison with ancillary information
+        # see https://github.com/astropy/astropy/pull/4522/files
+        return self.wcs.wcs.compare(other.wcs.wcs, cmp=1, tolerance=1e-6)
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -1025,16 +1080,3 @@ def world2pix(wcs, cdelt, crpix, coord):
         (pix[0] - (wcs.wcs.crpix[0] - 1.0)) * pix_ratio[0] + crpix[0] - 1.0,
         (pix[1] - (wcs.wcs.crpix[1] - 1.0)) * pix_ratio[1] + crpix[1] - 1.0,
     )
-
-
-def get_projection(wcs):
-    return wcs.wcs.ctype[0][5:]
-
-
-def get_coordys(wcs):
-    if "RA" in wcs.wcs.ctype[0]:
-        return "CEL"
-    elif "GLON" in wcs.wcs.ctype[0]:
-        return "GAL"
-    else:
-        raise ValueError("Unrecognized WCS coordinate system.")
