@@ -1,6 +1,9 @@
 print("import: 'gammapy'")
 import gammapy
 
+print("import: 'gammapy.analysis'")
+import gammapy.analysis
+
 print("import: 'gammapy.astro'")
 import gammapy.astro
 
